library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
library UNISIM;
use UNISIM.vcomponents.all;

entity AD936x_SPIInterface is
	port (	---------------------
				-- clock and reset --
				---------------------
				Clock_SPI		:  in std_logic;										
				Reset				:  in std_logic;										-- Reset, active low
				----------------------------------------------------
				-- AXI4 Stream Connection to Parameter Processing --
				----------------------------------------------------
				Rx_Valid			:  in std_logic;										
				Rx_Data			:  in std_logic_vector(31 downto 0);		
				Rx_Ack			: out std_logic;										
				Tx_Valid			: out std_logic;										
				Tx_Data			: out std_logic_vector(31 downto 0);			
				Tx_Ack			:  in std_logic;										
				-----------------------
				-- AD936x Interface  --
				-----------------------
				SPI_ENB			: out std_logic;									
				SPI_CLK			: out std_logic;										
				SPI_DO			:  in std_logic;										
				SPI_DI			: out std_logic);
end AD936x_SPIInterface;

architecture Behavioral of AD936x_SPIInterface is
signal SPI_CLK_En																: std_logic;								
type SPIStates is (Idle, SPI_0, SPI_1, SPI_2, SPI_3, SPI_4, SPI_5, SPI_6, SPI_7, SPI_8, SPI_9, SPI_10,
					    SPI_11, SPI_12, SPI_13, SPI_14, SPI_15, SPI_16, SPI_17, SPI_18, SPI_19, SPI_20,
						 SPI_21, SPI_22, SPI_23, SPI_24, SPI_25, SPI_26);
signal SPIState																: SPIStates;								
begin

	-- spi clock output
	CLK_Out: ODDR2 
	generic map	(	DDR_ALIGNMENT	=> "NONE",									
						INIT				=> '0',										
						SRTYPE			=> "SYNC")									
   port map		(	Q	=> SPI_CLK,  												
						C0 => Clock_SPI, 												
						C1	=> not Clock_SPI, 											
						CE	=> SPI_CLK_En, 											
						D0	=> '1', 														
						D1 => '0', 														
						R	=> not Reset,  											
						S	=> '0');														


	-- spi processing for data output
	DOut: process(Clock_SPI, Reset)
	begin
		if Reset='0' then	SPIState		<= Idle;
								SPI_DI		<= '0';
								Rx_Ack		<= '0';
								Tx_Valid 	<= '0';
											
		elsif rising_edge(Clock_SPI) then
			-- default --
			SPI_DI	<= '0';
			Rx_Ack	<= '0';
			-- reset flag --
			if Tx_Ack='1' then Tx_Valid <= '0'; end if;
			-- FSM --
			case SPIState is	
				when Idle	=>	if Rx_Valid='1' then SPIState <= SPI_0; end if;									
				when SPI_0	=> case Rx_Data(31 downto 24) is
										when x"01"	=>	SPI_DI 	<= '0';	SPIState <= SPI_1;	
										when x"02"	=> SPI_DI 	<= '1'; 	SPIState	<= SPI_1;	
										when others => 						SPIState	<= SPI_24; 	
									end case;
				when SPI_1	=> SPI_DI <= '0'; SPIState <= SPI_2;
				when SPI_2	=> SPI_DI <= '0'; SPIState <= SPI_3;
				when SPI_3	=> SPI_DI <= '0'; SPIState <= SPI_4;
				when SPI_4	=> SPI_DI <= '0'; SPIState <= SPI_5;
				when SPI_5	=> SPI_DI <= '0'; SPIState <= SPI_6;
				when SPI_6	=> SPI_DI <= Rx_Data(17); SPIState <= SPI_7;
				when SPI_7	=> SPI_DI <= Rx_Data(16); SPIState <= SPI_8;
				when SPI_8	=> SPI_DI <= Rx_Data(15); SPIState <= SPI_9;
				when SPI_9	=> SPI_DI <= Rx_Data(14); SPIState <= SPI_10;
				when SPI_10	=> SPI_DI <= Rx_Data(13); SPIState <= SPI_11;
				when SPI_11	=> SPI_DI <= Rx_Data(12); SPIState <= SPI_12;
				when SPI_12	=> SPI_DI <= Rx_Data(11); SPIState <= SPI_13;
				when SPI_13	=> SPI_DI <= Rx_Data(10); SPIState <= SPI_14;
				when SPI_14	=> SPI_DI <= Rx_Data(9); SPIState <= SPI_15;
				when SPI_15	=> SPI_DI <= Rx_Data(8); SPIState <= SPI_16;
				when SPI_16	=> SPI_DI <= Rx_Data(7); SPIState <= SPI_17;
				when SPI_17	=> SPI_DI <= Rx_Data(6); SPIState <= SPI_18;
				when SPI_18	=> SPI_DI <= Rx_Data(5); SPIState <= SPI_19;
				when SPI_19	=> SPI_DI <= Rx_Data(4); SPIState <= SPI_20;
				when SPI_20	=> SPI_DI <= Rx_Data(3); SPIState <= SPI_21;
				when SPI_21	=> SPI_DI <= Rx_Data(2); SPIState <= SPI_22;
				when SPI_22	=> SPI_DI <= Rx_Data(1); SPIState <= SPI_23;
				when SPI_23	=> SPI_DI <= Rx_Data(0); SPIState <= SPI_24; 
				when SPI_24	=> SPIState <= SPI_25;				
				when SPI_25	=> if Tx_Ack='1' then Tx_Valid <= '1'; Rx_Ack <= '1'; SPIState <= SPI_26; end if;
				when SPI_26	=> SPIState <= Idle;									
				when others => SPIState <= Idle;
			end case;
		end if;
	end process DOut;
	
	-- spi processing for data input
	DIn: process(Clock_SPI, Reset)
	begin
		if Reset='0' then Tx_Data 		<= (others => '0');
								SPI_CLK_En	<= '0';
											
		elsif falling_edge(Clock_SPI) then
			-- default --
			SPI_CLK_En <= '0';
			-- FSM --
			case SPIState is
				when SPI_0	=> Tx_Data <= Rx_Data;
				when SPI_17	=> Tx_Data(7) <= SPI_DO;
				when SPI_18	=> Tx_Data(6) <= SPI_DO;
				when SPI_19	=> Tx_Data(5) <= SPI_DO;
				when SPI_20	=> Tx_Data(4) <= SPI_DO;
				when SPI_21	=> Tx_Data(3) <= SPI_DO;
				when SPI_22	=> Tx_Data(2) <= SPI_DO;
				when SPI_23	=> Tx_Data(1) <= SPI_DO;
				when SPI_24	=> Tx_Data(0) <= SPI_DO;
				when others =>
			end case;
			-- set spi clock enable --
			if SPIState/=Idle and SPIState/=SPI_24 and SPIState/=SPI_25 and SPIState/=SPI_26 then SPI_CLK_En <= '1'; end if;
		end if;
	end process DIn;

	-- set spi enable
	SPI_ENB	<= not SPI_CLK_En;

end Behavioral;

