library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
library UNISIM;
use UNISIM.VComponents.all;

entity ClockManagement is
	port (	-----------------
				-- clock input --
				-----------------
				ExtClock_P		:  in std_logic;							
				ExtClock_N		:  in std_logic;							
				------------------
				-- clock output --
				------------------
				Reset_ALow		: out std_logic;							-- Reset, active low
				Clock_150		: out std_logic;							
				Clock_25			: out std_logic);							
end ClockManagement;

architecture Behavioral of ClockManagement is
signal Locked, Clock_100_Internal, Clock_FB, Clock_150_Pll, Clock_25_Pll : std_logic; -- Clock- und Resetsignale
begin

		--------------------
		-- generate reset --
		--------------------

		Reset_ALow		<= Locked;
	  
	  
		--------------------
		-- generate clock --
		--------------------
		
		-- input buffer
		CLKBUFIN: IBUFGDS
		port map		(	O  						=> Clock_100_Internal,					
							I  						=> ExtClock_P,							
							IB 						=> ExtClock_N);							

		-- pll
		SystemClock: PLL_BASE
		generic map	(	BANDWIDTH            => "OPTIMIZED",
							CLK_FEEDBACK         => "CLKFBOUT",
							COMPENSATION         => "SYSTEM_SYNCHRONOUS",
							DIVCLK_DIVIDE        => 1,
							CLKFBOUT_MULT        => 6,
							CLKFBOUT_PHASE       => 0.000,
							CLKOUT0_DIVIDE       => 4,
							CLKOUT0_PHASE        => 0.000,
							CLKOUT0_DUTY_CYCLE   => 0.500,
							CLKOUT1_DIVIDE       => 24,
							CLKOUT1_PHASE        => 0.000,
							CLKOUT1_DUTY_CYCLE   => 0.500,
							CLKIN_PERIOD         => 10.0,
							REF_JITTER           => 0.010)
		port map		(	CLKFBOUT            	=> Clock_FB,
							CLKOUT0             	=> Clock_150_Pll,   	
							CLKOUT1             	=> Clock_25_Pll,	
							CLKOUT2             	=> open,		
							CLKOUT3             	=> open,	
							CLKOUT4             	=> open,
							CLKOUT5             	=> open,
							LOCKED              	=> Locked,
							RST                 	=> '0',
							CLKFBIN             	=> Clock_FB,
							CLKIN               	=> Clock_100_Internal);
		
		-- output buffer
		T150Buf : BUFG
		port map		(	O   						=> Clock_150,
							I   						=> Clock_150_Pll);
							
		T25Buf : BUFG
		port map		(	O   						=> Clock_25,
							I   						=> Clock_25_Pll);
							

end Behavioral;