-----------------------------------
-- Project: Transceiver FMCOMMS4 --
-----------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
library UNISIM;
use UNISIM.vcomponents.all;

entity FMC_Connector_FPGA is
	port	(	------------------------
				-- differential clock --
				------------------------				
				ExtClock_P		:    in std_logic;								
				ExtClock_N		:    in std_logic;											
				---------------------------------------------------
				-- AXI4 Stream Connection to Parameter Interface --
				---------------------------------------------------				
				Ctrl_Rx_Clk		:    in std_logic;								
				Ctrl_Rx_Valid	:    in std_logic;								
				Ctrl_Rx_Data	:    in std_logic_vector(7 downto 0);		
				Ctrl_Rx_Ack		:   out std_logic;								
				Ctrl_Tx_Clk		:   out std_logic;								
				Ctrl_Tx_Valid	:   out std_logic;								
				Ctrl_Tx_Data	:   out std_logic_vector(7 downto 0);		
				Ctrl_Tx_Ack		:    in std_logic;								
				----------------------------------------------
				-- AXI4 Stream Data Connection to User FPGA --
				----------------------------------------------
				Data_Rx_Clk		:    in std_logic;								
				Data_Rx_Valid	:    in std_logic;								
				Data_Rx_Data	:    in std_logic_vector(15 downto 0);		
				Data_Rx_Ack		:   out std_logic;								
				Data_Tx_Clk		:	 out std_logic;								
				Data_Tx_Valid	:   out std_logic;								
				Data_Tx_Data	:   out std_logic_vector(15 downto 0);		
				Data_Tx_Ack		:    in std_logic;	
				--------------------------
				-- UART Debug Interface --
				--------------------------
				UART_RxD			:    in std_logic;								
				UART_TxD			:   out std_logic;	
				-------------------------
				-- FMC (LPC) Connector --
				-------------------------
				TRST_L			: inout std_logic;
				PRSNT_M2C_L		:    in std_logic;
				PG_C2M			:   out std_logic;
				DP0_C2M_P		:   out std_logic;
				DP0_C2M_N		:   out std_logic;
				DP0_M2C_P		:    in std_logic;
				DP0_M2C_N		:    in std_logic;
				CLK0_M2C_P		:    in std_logic;
				CLK0_M2C_N		:    in std_logic;
				GBTCLK0_M2C_P	:    in std_logic;
				GBTCLK0_M2C_N	:    in std_logic;
				CLK1_M2C_P		:    in std_logic;
				CLK1_M2C_N		:    in std_logic;
				LA00_P_CC		: inout std_logic;
				LA00_N_CC		: inout std_logic;
				LA01_P_CC		: inout std_logic;
				LA01_N_CC		: inout std_logic;
				LA02_P			: inout std_logic;
				LA02_N			: inout std_logic;
				LA03_P			: inout std_logic;
				LA03_N			: inout std_logic;
				LA04_P			: inout std_logic;
				LA04_N			: inout std_logic;
				LA05_P			: inout std_logic;
				LA05_N			: inout std_logic;
				LA06_P			: inout std_logic;
				LA06_N			: inout std_logic;
				LA07_P			: inout std_logic;
				LA07_N			: inout std_logic;
				LA08_P			: inout std_logic;
				LA08_N			: inout std_logic;
				LA09_P			: inout std_logic;
				LA09_N			: inout std_logic;
				LA10_P			: inout std_logic;
				LA10_N			: inout std_logic;
				LA11_P			: inout std_logic;
				LA11_N			: inout std_logic;
				LA12_P			: inout std_logic;
				LA12_N			: inout std_logic;
				LA13_P			: inout std_logic;
				LA13_N			: inout std_logic;
				LA14_P			: inout std_logic;
				LA14_N			: inout std_logic;
				LA15_P			: inout std_logic;
				LA15_N			: inout std_logic;
				LA16_P			: inout std_logic;
				LA16_N			: inout std_logic;
				LA17_P_CC		: inout std_logic;
				LA17_N_CC		: inout std_logic;
				LA18_P_CC		: inout std_logic;
				LA18_N_CC		: inout std_logic;
				LA19_P			: inout std_logic;
				LA19_N			: inout std_logic;
				LA20_P			: inout std_logic;
				LA20_N			: inout std_logic;
				LA21_P			: inout std_logic;
				LA21_N			: inout std_logic;
				LA22_P			: inout std_logic;
				LA22_N			: inout std_logic;
				LA23_P			: inout std_logic;
				LA23_N			: inout std_logic;
				LA24_P			: inout std_logic;
				LA24_N			: inout std_logic;
				LA25_P			: inout std_logic;
				LA25_N			: inout std_logic;
				LA26_P			: inout std_logic;
				LA26_N			: inout std_logic;
				LA27_P			: inout std_logic;
				LA27_N			: inout std_logic;
				LA28_P			: inout std_logic;
				LA28_N			: inout std_logic;
				LA29_P			: inout std_logic;
				LA29_N			: inout std_logic;
				LA30_P			: inout std_logic;
				LA30_N			: inout std_logic;
				LA31_P			: inout std_logic;
				LA31_N			: inout std_logic;
				LA32_P			: inout std_logic;
				LA32_N			: inout std_logic;
				LA33_P			: inout std_logic;
				LA33_N			: inout std_logic;
				SCL				: inout std_logic;		-- please solder serial resistor R703 first
				SDA				: inout std_logic);		-- please solder serial resistor R704 first		
end FMC_Connector_FPGA;

architecture Behavioral of FMC_Connector_FPGA is
signal Reset_ALow, Clock_150, Clock_25, Rx_Valid, Rx_Ack, Tx_Valid, Tx_Ack, SPI_IF_Rx_Valid, 
		 SPI_IF_Rx_Ack, SPI_IF_Tx_Valid, SPI_IF_Tx_Ack, AD936x_Enable, AD936x_ResetB, 
		 AD936x_Data_Clk, AD936x_FB_Clk, AD936x_Rx_Frame, AD936x_Tx_Frame, AD936x_TxNRx,
		 AD936x_En_AGC, AD936x_Sync_In, AD936x_ClkOut, AD936x_Rx_Valid, AD936x_Tx_Valid, 
		 AD936x_Tx_Ack, AD936x_Activated, AD936x_SPI_ENB, AD936x_SPI_DO, AD936x_SPI_DI,
		 AD936x_SPI_CLK, SPI_Rx_Valid, SPI_Rx_Ack, SPI_Tx_Valid, SPI_Tx_Ack, Samples_Rx_Valid,
		 Samples_Rx_Ack, Samples_Tx_Valid, Samples_Tx_Ack, Samples_FIFO_Enable, AD936x_Rx_Ack	: std_logic;
signal AD936x_Ctrl_In																								: std_logic_vector(3 downto 0);
signal Rx_Data, Tx_Data, AD936x_Ctrl_Out																		: std_logic_vector(7 downto 0);
signal AD936x_Data_Rx, AD936x_Data_Tx																			: std_logic_vector(11 downto 0);
signal AD936x_Rx_Data, AD936x_Tx_Data, SPI_IF_Rx_Data, SPI_IF_Tx_Data, SPI_Rx_Data, 
		 SPI_Tx_Data, Samples_Rx_Data, Samples_Tx_Data														: std_logic_vector(31 downto 0);
begin

	------------------------------
	-- clock & reset generation --
	------------------------------
	
	
	ClockGen: entity Work.ClockManagement
	port map	(	---------------------
					-- clock input --
					---------------------
					ExtClock_P			=> ExtClock_P,									
					ExtClock_N			=> ExtClock_N,									
					------------------
					-- clock output --
					------------------
					Reset_ALow			=> Reset_ALow,									-- Reset, active low
					Clock_150			=> Clock_150,									
					Clock_25				=> Clock_25);									
					

	-----------------
	-- FMC Wrapper --
	-----------------
	
	
	FMC_Wrapper: entity Work.AD936x_FMC_PinWrapper
	port map (	------------------------------
					-- FMC (LPC) Connector Pins --
					------------------------------
					TRST_L				=> TRST_L,
					PRSNT_M2C_L			=> PRSNT_M2C_L,
					PG_C2M				=> PG_C2M,
					DP0_C2M_P			=> DP0_C2M_P,
					DP0_C2M_N			=> DP0_C2M_N,
					DP0_M2C_P			=> DP0_M2C_P,
					DP0_M2C_N			=> DP0_M2C_N,
					CLK0_M2C_P			=> CLK0_M2C_P,
					CLK0_M2C_N			=> CLK0_M2C_N,
					GBTCLK0_M2C_P		=> GBTCLK0_M2C_P,
					GBTCLK0_M2C_N		=> GBTCLK0_M2C_N,
					CLK1_M2C_P			=> CLK1_M2C_P,
					CLK1_M2C_N			=> CLK1_M2C_N,
					LA00_P_CC			=> LA00_P_CC,
					LA00_N_CC			=> LA00_N_CC,
					LA01_P_CC			=> LA01_P_CC,
					LA01_N_CC			=> LA01_N_CC,
					LA02_P				=> LA02_P,
					LA02_N				=> LA02_N,
					LA03_P				=> LA03_P,
					LA03_N				=> LA03_N,
					LA04_P				=> LA04_P,
					LA04_N				=> LA04_N,
					LA05_P				=> LA05_P,
					LA05_N				=> LA05_N,
					LA06_P				=> LA06_P,
					LA06_N				=> LA06_N,
					LA07_P				=> LA07_P,
					LA07_N				=> LA07_N,
					LA08_P				=> LA08_P,
					LA08_N				=> LA08_N,
					LA09_P				=> LA09_P,
					LA09_N				=> LA09_N,
					LA10_P				=> LA10_P,
					LA10_N				=> LA10_N,
					LA11_P				=> LA11_P,
					LA11_N				=> LA11_N,
					LA12_P				=> LA12_P,
					LA12_N				=> LA12_N,
					LA13_P				=> LA13_P,
					LA13_N				=> LA13_N,
					LA14_P				=> LA14_P,
					LA14_N				=> LA14_N,
					LA15_P				=> LA15_P,
					LA15_N				=> LA15_N,
					LA16_P				=> LA16_P,
					LA16_N				=> LA16_N,
					LA17_P_CC			=> LA17_P_CC,
					LA17_N_CC			=> LA17_N_CC,
					LA18_P_CC			=> LA18_P_CC,
					LA18_N_CC			=> LA18_N_CC,
					LA19_P				=> LA19_P,
					LA19_N				=> LA19_N,
					LA20_P				=> LA20_P,
					LA20_N				=> LA20_N,
					LA21_P				=> LA21_P,
					LA21_N				=> LA21_N,
					LA22_P				=> LA22_P,
					LA22_N				=> LA22_N,
					LA23_P				=> LA23_P,
					LA23_N				=> LA23_N,
					LA24_P				=> LA24_P,
					LA24_N				=> LA24_N,
					LA25_P				=> LA25_P,
					LA25_N				=> LA25_N,
					LA26_P				=> LA26_P,
					LA26_N				=> LA26_N,
					LA27_P				=> LA27_P,
					LA27_N				=> LA27_N,
					LA28_P				=> LA28_P,
					LA28_N				=> LA28_N,
					LA29_P				=> LA29_P,
					LA29_N				=> LA29_N,
					LA30_P				=> LA30_P,
					LA30_N				=> LA30_N,
					LA31_P				=> LA31_P,
					LA31_N				=> LA31_N,
					LA32_P				=> LA32_P,
					LA32_N				=> LA32_N,
					LA33_P				=> LA33_P,
					LA33_N				=> LA33_N,
					SCL					=> SCL,
					SDA					=> SDA,
					---------------------
					-- control signals --
					---------------------
					-- status info --
					FMC_Plugged			=> open,								
					-- AD936x register interface --
					AD_CE					=> AD936x_SPI_ENB,							
					AD_MISO				=> AD936x_SPI_DO,								
					AD_MOSI				=> AD936x_SPI_DI,								
					AD_SCLK				=> AD936x_SPI_CLK,							
					-- AD936x data interface --
					AD_Enable			=> AD936x_Enable,								
					AD_ResetB			=> AD936x_ResetB,								
					AD_Data_Clk			=> AD936x_Data_Clk,  								
					AD_FB_Clk			=> AD936x_FB_Clk,								
					AD_Data_Rx			=> AD936x_Data_Rx,							
					AD_Data_Tx			=> AD936x_Data_Tx,							
					AD_Rx_Frame			=> AD936x_Rx_Frame,							
					AD_Tx_Frame			=> AD936x_Tx_Frame,							
					AD_TxNRx				=> AD936x_TxNRx,								
					AD_En_AGC			=> AD936x_En_AGC,								
					AD_Ctrl_Out			=> AD936x_Ctrl_Out,							
					AD_Ctrl_In			=> AD936x_Ctrl_In,     						
					AD_Sync_In			=> AD936x_Sync_In, 							
					AD_ClkOut			=> AD936x_ClkOut);							


	-----------------------------------
	-- Data Processing and Interface --
	-----------------------------------

	-- AD936x Control --
	AD936x_Data: entity Work.AD936x_IOInterface
	port map	(	---------------------
					-- clock and reset --
					---------------------	
					Clock						=> Clock_150,								
					Reset_ALow				=> Reset_ALow,								-- Reset, active low
					---------------------------------
					-- Interface to FMC PinWrapper --
					---------------------------------
					AD_Enable				=> AD936x_Enable,							
					AD_ResetB				=> AD936x_ResetB,							
					AD_Data_Clk				=> AD936x_Data_Clk,								
					AD_FB_Clk				=> AD936x_FB_Clk,							
					AD_Data_Rx				=> AD936x_Data_Rx,						
					AD_Data_Tx				=> AD936x_Data_Tx,						
					AD_Rx_Frame				=> AD936x_Rx_Frame,						
					AD_Tx_Frame				=> AD936x_Tx_Frame,						
					AD_TxNRx					=> AD936x_TxNRx,							
					AD_En_AGC				=> AD936x_En_AGC,							
					AD_Ctrl_Out				=> AD936x_Ctrl_Out,						
					AD_Ctrl_In				=> AD936x_Ctrl_In,						
					AD_Sync_In				=> AD936x_Sync_In,						
					AD_ClkOut				=> AD936x_ClkOut,							
					----------------------------------------------
					-- AXI4 Stream Connection to Data Interface --
					----------------------------------------------
					AD_Rx_Valid				=> AD936x_Rx_Valid,						
					AD_Rx_Data				=> AD936x_Rx_Data,
					AD_Rx_Ack				=> AD936x_Rx_Ack,
					AD_Tx_Valid				=> AD936x_Tx_Valid,						
					AD_Tx_Data				=> AD936x_Tx_Data,						
					AD_Tx_Ack				=> AD936x_Tx_Ack,	
					----------------------------------------------------
					-- AXI4 Stream Connection to Parameter Processing --
					----------------------------------------------------
					Samples_FIFO_Enable	=> Samples_FIFO_Enable,	
					Samples_Clk				=> Clock_25,
					Samples_Rx_Valid		=> Samples_Rx_Valid,								
					Samples_Rx_Data		=> Samples_Rx_Data,		
					Samples_Rx_Ack			=> Samples_Rx_Ack,									
					Samples_Tx_Valid		=> Samples_Tx_Valid,									
					Samples_Tx_Data		=> Samples_Tx_Data,		
					Samples_Tx_Ack			=> Samples_Tx_Ack,					
					---------------------
					-- control signals --
					---------------------
					AD936x_Activated		=> AD936x_Activated);					-- activate AD936x transceiver


	-- Data Interface to User FPGA --
	UserIF: entity Work.DataProcessing
	port map	(	---------------------
					-- clock and reset --
					---------------------
					Clock						=> Clock_150,									
					Reset_ALow				=> Reset_ALow,									-- Reset, active low
					----------------------------------------------
					-- AXI4 Stream Connection to AD936x Control --
					----------------------------------------------
					AD_Rx_Valid				=> AD936x_Rx_Valid,									
					AD_Rx_Data				=> AD936x_Rx_Data,
					AD_Rx_Ack				=> AD936x_Rx_Ack,
					AD_Tx_Valid				=> AD936x_Tx_Valid,									
					AD_Tx_Data				=> AD936x_Tx_Data,		
					AD_Tx_Ack				=> AD936x_Tx_Ack,									
					----------------------------------------------
					-- AXI4 Stream Data Connection to User FPGA --
					----------------------------------------------
					Data_Rx_Clk				=> Data_Rx_Clk,								
					Data_Rx_Valid			=> Data_Rx_Valid,								
					Data_Rx_Data			=> Data_Rx_Data,		
					Data_Rx_Ack				=> Data_Rx_Ack,							
					Data_Tx_Clk				=> Data_Tx_Clk,								
					Data_Tx_Valid			=> Data_Tx_Valid,								
					Data_Tx_Data			=> Data_Tx_Data,		
					Data_Tx_Ack				=> Data_Tx_Ack);									
				

	--------------------------
	-- Parameter Processing --
	--------------------------
	
	-- we use two small fifos for buffering the incoming and outgoing data --
	CtrlRxFIFO: entity Work.AXI4_FIFO_128x8
	port map		(	-- reset --
						s_aresetn				=> Reset_ALow,
						-- AXI4 Stream receive interface = write data to fifo --
						s_aclk					=> Ctrl_Rx_Clk,
						s_axis_tvalid			=> Ctrl_Rx_Valid,
						s_axis_tdata			=> Ctrl_Rx_Data,
						s_axis_tready			=> Ctrl_Rx_Ack,
						-- interface to signal processing process = read data from fifo --
						m_aclk					=> Clock_25,
						m_axis_tvalid			=> Rx_Valid,
						m_axis_tdata			=> Rx_Data,
						m_axis_tready			=> Rx_Ack);

	CtrlTxFIFO: entity Work.AXI4_FIFO_128x8
	port map		(	-- clock and reset --						
						s_aresetn				=> Reset_ALow,
						-- interface to signal processing process = write data to fifo --
						s_aclk					=> Clock_25,
						s_axis_tvalid			=> Tx_Valid,
						s_axis_tdata			=> Tx_Data,
						s_axis_tready			=> Tx_Ack,
						-- AXI4 Stream transit interface = read data from fifo --
						m_aclk					=> Clock_25,
						m_axis_tvalid			=> Ctrl_Tx_Valid,
						m_axis_tdata			=> Ctrl_Tx_Data,
						m_axis_tready			=> Ctrl_Tx_Ack);

	-- provide tx clock --
	CtrlClock: ODDR2 
	generic map	(	DDR_ALIGNMENT	=> "NONE",										-- Sets output alignment to "NONE", "C0", "C1"
						INIT				=> '0',											-- Sets initial state of the Q output to '0' or '1
						SRTYPE			=> "SYNC")										-- Specifies "SYNC" or "ASYNC" set/reset
   port map		(	Q	=> Ctrl_Tx_Clk,  												-- 1-bit DDR output data
						C0 => Clock_25, 													-- 1-bit clock input
						C1	=> not Clock_25, 												-- 1-bit clock input
						CE	=> '1', 															-- 1-bit clock enable input
						D0	=> '1', 															-- 1-bit data input (associated with C0)
						D1 => '0', 															-- 1-bit data input (associated with C1)
						R	=> not Reset_ALow,  											-- 1-bit reset input, active high
						S	=> '0');															-- 1-bit set input


	-- Parameter processing --
	Ctrl: entity Work.Protocol
	port map	(	---------------------
					-- clock and reset --
					---------------------			
					Clock						=> Clock_25,								
					Reset_ALow				=> Reset_ALow,								-- Reset,active low
					---------------------------------------------------
					-- AXI4 Stream Connection to Parameter Interface --
					---------------------------------------------------											
					Ctrl_Rx_Valid			=> Rx_Valid,								
					Ctrl_Rx_Data			=> Rx_Data,	
					Ctrl_Rx_Ack				=> Rx_Ack,															
					Ctrl_Tx_Valid			=> Tx_Valid,								
					Ctrl_Tx_Data			=> Tx_Data,		
					Ctrl_Tx_Ack				=> Tx_Ack,
					----------------------------------------------------
					-- AXI4 Stream Connection to AD936x SPI Interface --
					----------------------------------------------------
					SPI_Rx_Valid			=> SPI_Rx_Valid,							
					SPI_Rx_Data				=> SPI_Rx_Data,							
					SPI_Rx_Ack				=> SPI_Rx_Ack,								
					SPI_Tx_Valid			=> SPI_Tx_Valid,							
					SPI_Tx_Data				=> SPI_Tx_Data,							
					SPI_Tx_Ack				=> SPI_Tx_Ack,								
					-----------------------------------------------------
					-- AXI4 Stream Connection to AD936x Data Interface --
					-----------------------------------------------------
					Samples_FIFO_Enable	=> Samples_FIFO_Enable,					
					Samples_Rx_Valid		=> Samples_Rx_Valid,						
					Samples_Rx_Data		=> Samples_Rx_Data,						
					Samples_Rx_Ack			=> Samples_Rx_Ack,						
					Samples_Tx_Valid		=> Samples_Tx_Valid,						
					Samples_Tx_Data		=> Samples_Tx_Data,						
					Samples_Tx_Ack			=> Samples_Tx_Ack,						
					---------------------
					-- control signals --
					---------------------
					AD936x_Activated		=> AD936x_Activated);					-- (de)activate AD936x transceiver


	--------------------------
	-- AD936x SPI Interface --
	--------------------------

	-- we use two small fifos for buffering the incoming and outgoing data --
	SPIRxFIFO: entity Work.AXI4_FIFO_16x32
	port map		(	-- reset --
						s_aresetn				=> Reset_ALow,
						-- AXI4 Stream receive interface = write data to fifo --
						s_aclk					=> Clock_25,
						s_axis_tvalid			=> SPI_IF_Tx_Valid,
						s_axis_tdata			=> SPI_IF_Tx_Data,
						s_axis_tready			=> SPI_IF_Tx_Ack,
						-- interface to signal processing process = read data from fifo --
						m_aclk					=> Clock_25,
						m_axis_tvalid			=> SPI_Rx_Valid,
						m_axis_tdata			=> SPI_Rx_Data,
						m_axis_tready			=> SPI_Rx_Ack);

	SPITxFIFO: entity Work.AXI4_FIFO_16x32
	port map		(	-- clock and reset --						
						s_aresetn				=> Reset_ALow,
						-- interface to signal processing process = write data to fifo --
						s_aclk					=> Clock_25,
						s_axis_tvalid			=> SPI_Tx_Valid,
						s_axis_tdata			=> SPI_Tx_Data,
						s_axis_tready			=> SPI_Tx_Ack,
						-- AXI4 Stream transit interface = read data from fifo --
						m_aclk					=> Clock_25,
						m_axis_tvalid			=> SPI_IF_Rx_Valid,
						m_axis_tdata			=> SPI_IF_Rx_Data,
						m_axis_tready			=> SPI_IF_Rx_Ack);
						
	-- SPI Interface --
	SPI: entity Work.AD936x_SPIInterface
	port map (	---------------------
					-- clock and reset --
					---------------------
					Clock_SPI			=> Clock_25,										
					Reset					=> Reset_ALow,									-- Reset, active low
					----------------------------------------------------
					-- AXI4 Stream Connection to Parameter Processing --
					----------------------------------------------------
					Rx_Valid				=> SPI_IF_Rx_Valid,								
					Rx_Data				=> SPI_IF_Rx_Data,							
					Rx_Ack				=> SPI_IF_Rx_Ack,									
					Tx_Valid				=> SPI_IF_Tx_Valid,								
					Tx_Data				=> SPI_IF_Tx_Data,								
					Tx_Ack				=> SPI_IF_Tx_Ack,									
					-----------------------
					-- AD936x Interface  --
					-----------------------
					SPI_ENB				=> AD936x_SPI_ENB,							
					SPI_CLK				=> AD936x_SPI_CLK,							
					SPI_DO				=> AD936x_SPI_DO,								
					SPI_DI				=> AD936x_SPI_DI);


	--------------
	-- not used --
	--------------
	
	UART_TxD <= '1';

end Behavioral;